@@ -10,6 +10,8 @@ Methods for optimizing acquisition functions.
 
 from __future__ import annotations
 
+import dataclasses
+
 import time
 import warnings
 
@@ -23,13 +25,15 @@ from botorch.acquisition.acquisition import (
 from botorch.acquisition.knowledge_gradient import qKnowledgeGradient
 from botorch.exceptions import InputDataError, UnsupportedError
 from botorch.exceptions.warnings import OptimizationWarning
-from botorch.generation.gen import gen_candidates_scipy
+from botorch.generation.gen import gen_candidates_scipy, TGenCandidates
 from botorch.logging import logger
 from botorch.optim.initializers import (
     gen_batch_initial_conditions,
     gen_one_shot_kg_initial_conditions,
+    TGenInitialConditions,
 )
 from botorch.optim.stopping import ExpMAStoppingCriterion
+from botorch.optim.utils import _filter_kwargs
 from torch import Tensor
 
 INIT_OPTION_KEYS = {
@@ -45,12 +49,350 @@ INIT_OPTION_KEYS = {
     "sample_around_best",
     "sample_around_best_sigma",
     "sample_around_best_prob_perturb",
-    "sample_around_best_prob_perturb",
     "seed",
     "thinning",
 }
 
 
+@dataclasses.dataclass(frozen=True)
+class OptimizeAcqfInputs:
+    """
+    Container for inputs to `optimize_acqf`.
+
+    See docstring for `optimize_acqf` for explanation of parameters.
+    """
+
+    acq_function: AcquisitionFunction
+    bounds: Tensor
+    q: int
+    num_restarts: int
+    raw_samples: Optional[int]
+    options: Optional[Dict[str, Union[bool, float, int, str]]]
+    inequality_constraints: Optional[List[Tuple[Tensor, Tensor, float]]]
+    equality_constraints: Optional[List[Tuple[Tensor, Tensor, float]]]
+    nonlinear_inequality_constraints: Optional[List[Callable]]
+    fixed_features: Optional[Dict[int, float]]
+    post_processing_func: Optional[Callable[[Tensor], Tensor]]
+    batch_initial_conditions: Optional[Tensor]
+    return_best_only: bool
+    gen_candidates: TGenCandidates
+    sequential: bool
+    ic_generator: Optional[TGenInitialConditions] = None
+    timeout_sec: Optional[float] = None
+    return_full_tree: bool = False
+    retry_on_optimization_warning: bool = True
+    ic_gen_kwargs: Dict = dataclasses.field(default_factory=dict)
+
+    @property
+    def full_tree(self) -> bool:
+        return self.return_full_tree or (
+            not isinstance(self.acq_function, OneShotAcquisitionFunction)
+        )
+
+    def __post_init__(self) -> None:
+        if self.inequality_constraints is None and not (
+            self.bounds.ndim == 2 and self.bounds.shape[0] == 2
+        ):
+            raise ValueError(
+                "bounds should be a `2 x d` tensor, current shape: "
+                f"{list(self.bounds.shape)}."
+            )
+
+        # TODO: Validate constraints if provided:
+        # https://github.com/pytorch/botorch/pull/1231
+        if self.batch_initial_conditions is not None and self.sequential:
+            raise UnsupportedError(
+                "`batch_initial_conditions` is not supported for sequential "
+                "optimization. Either avoid specifying "
+                "`batch_initial_conditions` to use the custom initializer or "
+                "use the `ic_generator` kwarg to generate initial conditions "
+                "for the case of nonlinear inequality constraints."
+            )
+
+        d = self.bounds.shape[1]
+        if self.batch_initial_conditions is not None:
+            batch_initial_conditions_shape = self.batch_initial_conditions.shape
+            if len(batch_initial_conditions_shape) not in (2, 3):
+                raise ValueError(
+                    "batch_initial_conditions must be 2-dimensional or "
+                    "3-dimensional. Its shape is "
+                    f"{batch_initial_conditions_shape}."
+                )
+            if batch_initial_conditions_shape[-1] != d:
+                raise ValueError(
+                    f"batch_initial_conditions.shape[-1] must be {d}. The "
+                    f"shape is {batch_initial_conditions_shape}."
+                )
+
+        elif self.ic_generator is None:
+            if self.nonlinear_inequality_constraints is not None:
+                raise RuntimeError(
+                    "`ic_generator` must be given if "
+                    "there are non-linear inequality constraints."
+                )
+            if self.raw_samples is None:
+                raise ValueError(
+                    "Must specify `raw_samples` when "
+                    "`batch_initial_conditions` is None`."
+                )
+
+        if self.sequential and self.q > 1:
+            if not self.return_best_only:
+                raise NotImplementedError(
+                    "`return_best_only=False` only supported for joint optimization."
+                )
+            if isinstance(self.acq_function, OneShotAcquisitionFunction):
+                raise NotImplementedError(
+                    "sequential optimization currently not supported for one-shot "
+                    "acquisition functions. Must have `sequential=False`."
+                )
+
+    def get_ic_generator(self) -> TGenInitialConditions:
+        if self.ic_generator is not None:
+            return self.ic_generator
+        elif isinstance(self.acq_function, qKnowledgeGradient):
+            return gen_one_shot_kg_initial_conditions
+        return gen_batch_initial_conditions
+
+
+def _raise_deprecation_warning_if_kwargs(fn_name: str, kwargs: Dict[str, Any]) -> None:
+    """
+    Raise a warning if kwargs are provided.
+
+    Some functions used to support **kwargs. The applicable parameters have now been
+    refactored to be named arguments, so no warning will be raised for users passing
+    the expected arguments. However, if a user had been passing an inapplicable
+    keyword argument, this will now raise a warning whereas in the past it did
+    nothing.
+    """
+    if len(kwargs) > 0:
+        warnings.warn(
+            f"`{fn_name}` does not support arguments {list(kwargs.keys())}. In "
+            "the future, this will become an error.",
+            DeprecationWarning,
+        )
+
+
+def _optimize_acqf_all_features_fixed(
+    *,
+    bounds: Tensor,
+    fixed_features: Dict[int, float],
+    q: int,
+    acq_function: AcquisitionFunction,
+) -> Tuple[Tensor, Tensor]:
+    """
+    Helper function for `optimize_acqf` for the trivial case where
+    all features are fixed.
+    """
+    X = torch.tensor(
+        [fixed_features[i] for i in range(bounds.shape[-1])],
+        device=bounds.device,
+        dtype=bounds.dtype,
+    )
+    X = X.expand(q, *X.shape)
+    with torch.no_grad():
+        acq_value = acq_function(X)
+    return X, acq_value
+
+
+def _optimize_acqf_sequential_q(
+    opt_inputs: OptimizeAcqfInputs, timeout_sec: Optional[float], start_time: float
+) -> Tuple[Tensor, Tensor]:
+    """
+    Helper function for `optimize_acqf` when sequential=True and q > 1.
+    """
+    if timeout_sec is not None:
+        # When using sequential optimization, we allocate the total timeout
+        # evenly across the individual acquisition optimizations.
+        timeout_sec = (timeout_sec - start_time) / opt_inputs.q
+
+    candidate_list, acq_value_list = [], []
+    base_X_pending = opt_inputs.acq_function.X_pending
+
+    new_inputs = dataclasses.replace(
+        opt_inputs,
+        q=1,
+        batch_initial_conditions=None,
+        return_best_only=True,
+        sequential=False,
+        timeout_sec=timeout_sec,
+    )
+    for i in range(opt_inputs.q):
+
+        candidate, acq_value = _optimize_acqf_batch(
+            new_inputs, start_time=start_time, timeout_sec=timeout_sec
+        )
+
+        candidate_list.append(candidate)
+        acq_value_list.append(acq_value)
+        candidates = torch.cat(candidate_list, dim=-2)
+        new_inputs.acq_function.set_X_pending(
+            torch.cat([base_X_pending, candidates], dim=-2)
+            if base_X_pending is not None
+            else candidates
+        )
+        logger.info(f"Generated sequential candidate {i+1} of {opt_inputs.q}")
+    opt_inputs.acq_function.set_X_pending(base_X_pending)
+    return candidates, torch.stack(acq_value_list)
+
+
+def _optimize_acqf_batch(
+    opt_inputs: OptimizeAcqfInputs, start_time: float, timeout_sec: Optional[float]
+) -> Tuple[Tensor, Tensor]:
+    options = opt_inputs.options or {}
+
+    initial_conditions_provided = opt_inputs.batch_initial_conditions is not None
+
+    if initial_conditions_provided:
+        batch_initial_conditions = opt_inputs.batch_initial_conditions
+    else:
+        # pyre-ignore[28]: Unexpected keyword argument `acq_function` to anonymous call.
+        batch_initial_conditions = opt_inputs.get_ic_generator()(
+            acq_function=opt_inputs.acq_function,
+            bounds=opt_inputs.bounds,
+            q=opt_inputs.q,
+            num_restarts=opt_inputs.num_restarts,
+            raw_samples=opt_inputs.raw_samples,
+            fixed_features=opt_inputs.fixed_features,
+            options=options,
+            inequality_constraints=opt_inputs.inequality_constraints,
+            equality_constraints=opt_inputs.equality_constraints,
+            **opt_inputs.ic_gen_kwargs,
+        )
+
+    batch_limit: int = options.get(
+        "batch_limit",
+        opt_inputs.num_restarts
+        if not opt_inputs.nonlinear_inequality_constraints
+        else 1,
+    )
+    has_parameter_constraints = (
+        opt_inputs.inequality_constraints is not None
+        or opt_inputs.equality_constraints is not None
+        or opt_inputs.nonlinear_inequality_constraints is not None
+    )
+
+    def _optimize_batch_candidates(
+        timeout_sec: Optional[float],
+    ) -> Tuple[Tensor, Tensor, List[Warning]]:
+        batch_candidates_list: List[Tensor] = []
+        batch_acq_values_list: List[Tensor] = []
+        batched_ics = batch_initial_conditions.split(batch_limit)
+        opt_warnings = []
+        if timeout_sec is not None:
+            timeout_sec = (timeout_sec - start_time) / len(batched_ics)
+
+        bounds = opt_inputs.bounds
+        gen_kwargs: Dict[str, Any] = {
+            "lower_bounds": None if bounds[0].isinf().all() else bounds[0],
+            "upper_bounds": None if bounds[1].isinf().all() else bounds[1],
+            "options": {k: v for k, v in options.items() if k not in INIT_OPTION_KEYS},
+            "fixed_features": opt_inputs.fixed_features,
+            "timeout_sec": timeout_sec,
+        }
+
+        if has_parameter_constraints:
+            # only add parameter constraints to gen_kwargs if they are specified
+            # to avoid unnecessary warnings in _filter_kwargs
+            gen_kwargs.update(
+                {
+                    "inequality_constraints": opt_inputs.inequality_constraints,
+                    "equality_constraints": opt_inputs.equality_constraints,
+                    # the line is too long
+                    "nonlinear_inequality_constraints": (
+                        opt_inputs.nonlinear_inequality_constraints
+                    ),
+                }
+            )
+        filtered_gen_kwargs = _filter_kwargs(opt_inputs.gen_candidates, **gen_kwargs)
+
+        for i, batched_ics_ in enumerate(batched_ics):
+            # optimize using random restart optimization
+            with warnings.catch_warnings(record=True) as ws:
+                warnings.simplefilter("always", category=OptimizationWarning)
+                (
+                    batch_candidates_curr,
+                    batch_acq_values_curr,
+                ) = opt_inputs.gen_candidates(
+                    batched_ics_, opt_inputs.acq_function, **filtered_gen_kwargs
+                )
+            opt_warnings += ws
+            batch_candidates_list.append(batch_candidates_curr)
+            batch_acq_values_list.append(batch_acq_values_curr)
+            logger.info(f"Generated candidate batch {i+1} of {len(batched_ics)}.")
+
+        batch_candidates = torch.cat(batch_candidates_list)
+        has_scalars = batch_acq_values_list[0].ndim == 0
+        if has_scalars:
+            batch_acq_values = torch.stack(batch_acq_values_list)
+        else:
+            batch_acq_values = torch.cat(batch_acq_values_list).flatten()
+        return batch_candidates, batch_acq_values, opt_warnings
+
+    batch_candidates, batch_acq_values, ws = _optimize_batch_candidates(timeout_sec)
+
+    optimization_warning_raised = any(
+        (issubclass(w.category, OptimizationWarning) for w in ws)
+    )
+    if optimization_warning_raised and opt_inputs.retry_on_optimization_warning:
+        first_warn_msg = (
+            "Optimization failed in `gen_candidates_scipy` with the following "
+            f"warning(s):\n{[w.message for w in ws]}\nBecause you specified "
+            "`batch_initial_conditions`, optimization will not be retried with "
+            "new initial conditions and will proceed with the current solution."
+            " Suggested remediation: Try again with different "
+            "`batch_initial_conditions`, or don't provide `batch_initial_conditions.`"
+            if initial_conditions_provided
+            else "Optimization failed in `gen_candidates_scipy` with the following "
+            f"warning(s):\n{[w.message for w in ws]}\nTrying again with a new "
+            "set of initial conditions."
+        )
+        warnings.warn(first_warn_msg, RuntimeWarning)
+
+        if not initial_conditions_provided:
+            batch_initial_conditions = opt_inputs.get_ic_generator()(
+                acq_function=opt_inputs.acq_function,
+                bounds=opt_inputs.bounds,
+                q=opt_inputs.q,
+                num_restarts=opt_inputs.num_restarts,
+                raw_samples=opt_inputs.raw_samples,
+                fixed_features=opt_inputs.fixed_features,
+                options=options,
+                inequality_constraints=opt_inputs.inequality_constraints,
+                equality_constraints=opt_inputs.equality_constraints,
+                **opt_inputs.ic_gen_kwargs,
+            )
+
+            batch_candidates, batch_acq_values, ws = _optimize_batch_candidates(
+                timeout_sec
+            )
+
+            optimization_warning_raised = any(
+                (issubclass(w.category, OptimizationWarning) for w in ws)
+            )
+            if optimization_warning_raised:
+                warnings.warn(
+                    "Optimization failed on the second try, after generating a "
+                    "new set of initial conditions.",
+                    RuntimeWarning,
+                )
+
+    if opt_inputs.post_processing_func is not None:
+        batch_candidates = opt_inputs.post_processing_func(batch_candidates)
+
+    if opt_inputs.return_best_only:
+        best = torch.argmax(batch_acq_values.view(-1), dim=0)
+        batch_candidates = batch_candidates[best]
+        batch_acq_values = batch_acq_values[best]
+
+    if not opt_inputs.full_tree:
+        batch_candidates = opt_inputs.acq_function.extract_candidates(
+            X_full=batch_candidates
+        )
+
+    return batch_candidates, batch_acq_values
+
+
 def optimize_acqf(
     acq_function: AcquisitionFunction,
     bounds: Tensor,
@@ -65,8 +407,14 @@ def optimize_acqf(
     post_processing_func: Optional[Callable[[Tensor], Tensor]] = None,
     batch_initial_conditions: Optional[Tensor] = None,
     return_best_only: bool = True,
+    gen_candidates: Optional[TGenCandidates] = None,
     sequential: bool = False,
-    **kwargs: Any,
+    *,
+    ic_generator: Optional[TGenInitialConditions] = None,
+    timeout_sec: Optional[float] = None,
+    return_full_tree: bool = False,
+    retry_on_optimization_warning: bool = True,
+    **ic_gen_kwargs: Any,
 ) -> Tuple[Tensor, Tensor]:
     r"""Generate a set of candidates via multi-start optimization.
 
@@ -104,14 +452,32 @@ def optimize_acqf(
             this if you do not want to use default initialization strategy.
         return_best_only: If False, outputs the solutions corresponding to all
             random restart initializations of the optimization.
+        gen_candidates: A callable for generating candidates (and their associated
+            acquisition values) given a tensor of initial conditions and an
+            acquisition function. Other common inputs include lower and upper bounds
+            and a dictionary of options, but refer to the documentation of specific
+            generation functions (e.g gen_candidates_scipy and gen_candidates_torch)
+            for method-specific inputs. Default: `gen_candidates_scipy`
         sequential: If False, uses joint optimization, otherwise uses sequential
             optimization.
-        kwargs: Additonal keyword arguments.
+        ic_generator: Function for generating initial conditions. Not needed when
+            `batch_initial_conditions` are provided. Defaults to
+            `gen_one_shot_kg_initial_conditions` for `qKnowledgeGradient` acquisition
+            functions and `gen_batch_initial_conditions` otherwise. Must be specified
+            for nonlinear inequality constraints.
+        timeout_sec: Max amount of time optimization can run for.
+        return_full_tree:
+        retry_on_optimization_warning: Whether to retry candidate generation with a new
+            set of initial conditions when it fails with an `OptimizationWarning`.
+        ic_gen_kwargs: Additional keyword arguments passed to function specified by
+            `ic_generator`
 
     Returns:
         A two-element tuple containing
 
-        - a `(num_restarts) x q x d`-dim tensor of generated candidates.
+        - A tensor of generated candidates. The shape is
+            -- `q x d` if `return_best_only` is True (default)
+            -- `num_restarts x q x d` if `return_best_only` is False
         - a tensor of associated acquisition values. If `sequential=False`,
             this is a `(num_restarts)`-dim tensor of joint acquisition values
             (with explicit restart dimension if `return_best_only=False`). If
@@ -131,238 +497,64 @@ def optimize_acqf(
         >>>     qEI, bounds, 3, 15, 256, sequential=True
         >>> )
     """
-    start_time: float = time.monotonic()
-    timeout_sec = kwargs.pop("timeout_sec", None)
+    # using a default of None simplifies unit testing
+    if gen_candidates is None:
+        gen_candidates = gen_candidates_scipy
+    opt_acqf_inputs = OptimizeAcqfInputs(
+        acq_function=acq_function,
+        bounds=bounds,
+        q=q,
+        num_restarts=num_restarts,
+        raw_samples=raw_samples,
+        options=options,
+        inequality_constraints=inequality_constraints,
+        equality_constraints=equality_constraints,
+        nonlinear_inequality_constraints=nonlinear_inequality_constraints,
+        fixed_features=fixed_features,
+        post_processing_func=post_processing_func,
+        batch_initial_conditions=batch_initial_conditions,
+        return_best_only=return_best_only,
+        gen_candidates=gen_candidates,
+        sequential=sequential,
+        ic_generator=ic_generator,
+        timeout_sec=timeout_sec,
+        return_full_tree=return_full_tree,
+        retry_on_optimization_warning=retry_on_optimization_warning,
+        ic_gen_kwargs=ic_gen_kwargs,
+    )
+    return _optimize_acqf(opt_acqf_inputs)
 
-    if inequality_constraints is None:
-        if not (bounds.ndim == 2 and bounds.shape[0] == 2):
-            raise ValueError(
-                "bounds should be a `2 x d` tensor, current shape: "
-                f"{list(bounds.shape)}."
-            )
-        # TODO: Validate constraints if provided:
-        # https://github.com/pytorch/botorch/pull/1231
 
-    full_tree = False
-
-    if isinstance(acq_function, OneShotAcquisitionFunction):
-        if not kwargs.pop("return_full_tree", False):
-            full_tree = True
-
-    initial_conditions_provided = batch_initial_conditions is not None
-
-    if initial_conditions_provided and sequential:
-        raise UnsupportedError(
-            "`batch_initial_conditions` is not supported for sequential optimization. "
-            "Either avoid specifying `batch_initial_conditions` to use the custom "
-            "initializer or use the `ic_generator` kwarg to generate "
-            "initial conditions for the case of nonlinear inequality constraints."
-        )
-
-    # Sets initial condition generator ic_gen if initial conditions not provided
-    if not initial_conditions_provided:
-        ic_gen = kwargs.pop("ic_generator", None)
-        if ic_gen is None:
-            if nonlinear_inequality_constraints:
-                raise RuntimeError(
-                    "`ic_generator` must be given if "
-                    "there are non-linear inequality constraints."
-                )
-            if raw_samples is None:
-                raise ValueError(
-                    "Must specify `raw_samples` when "
-                    "`batch_initial_conditions` is None`."
-                )
-            ic_gen = (
-                gen_one_shot_kg_initial_conditions
-                if isinstance(acq_function, qKnowledgeGradient)
-                else gen_batch_initial_conditions
-            )
-
-    # Perform sequential optimization via successive conditioning on pending points
-    if sequential and q > 1:
-        if timeout_sec is not None:
-            # When using sequential optimization, we allocate the total timeout
-            # evenly across the individual acquisition optimizations.
-            timeout_sec = (timeout_sec - start_time) / q
-        if not return_best_only:
-            raise NotImplementedError(
-                "`return_best_only=False` only supported for joint optimization."
-            )
-        if isinstance(acq_function, OneShotAcquisitionFunction):
-            raise NotImplementedError(
-                "sequential optimization currently not supported for one-shot "
-                "acquisition functions. Must have `sequential=False`."
-            )
-        candidate_list, acq_value_list = [], []
-        base_X_pending = acq_function.X_pending
-        for i in range(q):
-
-            candidate, acq_value = optimize_acqf(
-                acq_function=acq_function,
-                bounds=bounds,
-                q=1,
-                num_restarts=num_restarts,
-                raw_samples=raw_samples,
-                options=options or {},
-                inequality_constraints=inequality_constraints,
-                equality_constraints=equality_constraints,
-                nonlinear_inequality_constraints=nonlinear_inequality_constraints,
-                fixed_features=fixed_features,
-                post_processing_func=post_processing_func,
-                batch_initial_conditions=None,
-                return_best_only=True,
-                sequential=False,
-                ic_generator=ic_gen,
-                timeout_sec=timeout_sec,
-            )
-
-            candidate_list.append(candidate)
-            acq_value_list.append(acq_value)
-            candidates = torch.cat(candidate_list, dim=-2)
-            acq_function.set_X_pending(
-                torch.cat([base_X_pending, candidates], dim=-2)
-                if base_X_pending is not None
-                else candidates
-            )
-            logger.info(f"Generated sequential candidate {i+1} of {q}")
-        # Reset acq_func to previous X_pending state
-        acq_function.set_X_pending(base_X_pending)
-        return candidates, torch.stack(acq_value_list)
-
-    # Batch optimization (including the case q=1)
-    options = options or {}
+def _optimize_acqf(opt_inputs: OptimizeAcqfInputs) -> Tuple[Tensor, Tensor]:
 
     # Handle the trivial case when all features are fixed
-    if fixed_features is not None and len(fixed_features) == bounds.shape[-1]:
-        X = torch.tensor(
-            [fixed_features[i] for i in range(bounds.shape[-1])],
-            device=bounds.device,
-            dtype=bounds.dtype,
-        )
-        X = X.expand(q, *X.shape)
-        with torch.no_grad():
-            acq_value = acq_function(X)
-        return X, acq_value
-
-    if not initial_conditions_provided:
-
-        batch_initial_conditions = ic_gen(
-            acq_function=acq_function,
-            bounds=bounds,
-            q=q,
-            num_restarts=num_restarts,
-            raw_samples=raw_samples,
-            fixed_features=fixed_features,
-            options=options,
-            inequality_constraints=inequality_constraints,
-            equality_constraints=equality_constraints,
-            **kwargs,
+    if (
+        opt_inputs.fixed_features is not None
+        and len(opt_inputs.fixed_features) == opt_inputs.bounds.shape[-1]
+    ):
+        return _optimize_acqf_all_features_fixed(
+            bounds=opt_inputs.bounds,
+            fixed_features=opt_inputs.fixed_features,
+            q=opt_inputs.q,
+            acq_function=opt_inputs.acq_function,
         )
 
-    batch_limit: int = options.get(
-        "batch_limit", num_restarts if not nonlinear_inequality_constraints else 1
+    start_time: float = time.monotonic()
+    timeout_sec = opt_inputs.timeout_sec
+
+    # Perform sequential optimization via successive conditioning on pending points
+    if opt_inputs.sequential and opt_inputs.q > 1:
+        return _optimize_acqf_sequential_q(
+            opt_inputs=opt_inputs,
+            timeout_sec=timeout_sec,
+            start_time=start_time,
+        )
+
+    # Batch optimization (including the case q=1)
+    return _optimize_acqf_batch(
+        opt_inputs=opt_inputs, start_time=start_time, timeout_sec=timeout_sec
     )
 
-    def _optimize_batch_candidates(
-        timeout_sec: Optional[float],
-    ) -> Tuple[Tensor, Tensor, List[Warning]]:
-        batch_candidates_list: List[Tensor] = []
-        batch_acq_values_list: List[Tensor] = []
-        batched_ics = batch_initial_conditions.split(batch_limit)
-        opt_warnings = []
-        if timeout_sec is not None:
-            timeout_sec = (timeout_sec - start_time) / len(batched_ics)
-
-        scipy_kws = {
-            "acquisition_function": acq_function,
-            "lower_bounds": None if bounds[0].isinf().all() else bounds[0],
-            "upper_bounds": None if bounds[1].isinf().all() else bounds[1],
-            "options": {k: v for k, v in options.items() if k not in INIT_OPTION_KEYS},
-            "inequality_constraints": inequality_constraints,
-            "equality_constraints": equality_constraints,
-            "nonlinear_inequality_constraints": nonlinear_inequality_constraints,
-            "fixed_features": fixed_features,
-            "timeout_sec": timeout_sec,
-        }
-
-        for i, batched_ics_ in enumerate(batched_ics):
-            # optimize using random restart optimization
-            with warnings.catch_warnings(record=True) as ws:
-                warnings.simplefilter("always", category=OptimizationWarning)
-                batch_candidates_curr, batch_acq_values_curr = gen_candidates_scipy(
-                    initial_conditions=batched_ics_, **scipy_kws
-                )
-            opt_warnings += ws
-            batch_candidates_list.append(batch_candidates_curr)
-            batch_acq_values_list.append(batch_acq_values_curr)
-            logger.info(f"Generated candidate batch {i+1} of {len(batched_ics)}.")
-
-        batch_candidates = torch.cat(batch_candidates_list)
-        batch_acq_values = torch.cat(batch_acq_values_list)
-        return batch_candidates, batch_acq_values, opt_warnings
-
-    batch_candidates, batch_acq_values, ws = _optimize_batch_candidates(timeout_sec)
-
-    optimization_warning_raised = any(
-        (issubclass(w.category, OptimizationWarning) for w in ws)
-    )
-    if optimization_warning_raised:
-        first_warn_msg = (
-            "Optimization failed in `gen_candidates_scipy` with the following "
-            f"warning(s):\n{[w.message for w in ws]}\nBecause you specified "
-            "`batch_initial_conditions`, optimization will not be retried with "
-            "new initial conditions and will proceed with the current solution."
-            " Suggested remediation: Try again with different "
-            "`batch_initial_conditions`, or don't provide `batch_initial_conditions.`"
-            if initial_conditions_provided
-            else "Optimization failed in `gen_candidates_scipy` with the following "
-            f"warning(s):\n{[w.message for w in ws]}\nTrying again with a new "
-            "set of initial conditions."
-        )
-        warnings.warn(first_warn_msg, RuntimeWarning)
-
-        if not initial_conditions_provided:
-            batch_initial_conditions = ic_gen(
-                acq_function=acq_function,
-                bounds=bounds,
-                q=q,
-                num_restarts=num_restarts,
-                raw_samples=raw_samples,
-                fixed_features=fixed_features,
-                options=options,
-                inequality_constraints=inequality_constraints,
-                equality_constraints=equality_constraints,
-                **kwargs,
-            )
-
-            batch_candidates, batch_acq_values, ws = _optimize_batch_candidates(
-                timeout_sec
-            )
-
-            optimization_warning_raised = any(
-                (issubclass(w.category, OptimizationWarning) for w in ws)
-            )
-            if optimization_warning_raised:
-                warnings.warn(
-                    "Optimization failed on the second try, after generating a "
-                    "new set of initial conditions.",
-                    RuntimeWarning,
-                )
-
-    if post_processing_func is not None:
-        batch_candidates = post_processing_func(batch_candidates)
-
-    if return_best_only:
-        best = torch.argmax(batch_acq_values.view(-1), dim=0)
-        batch_candidates = batch_candidates[best]
-        batch_acq_values = batch_acq_values[best]
-
-    if full_tree:
-        batch_candidates = acq_function.extract_candidates(X_full=batch_candidates)
-
-    return batch_candidates, batch_acq_values
-
 
 def optimize_acqf_cyclic(
     acq_function: AcquisitionFunction,
@@ -377,7 +569,12 @@ def optimize_acqf_cyclic(
     post_processing_func: Optional[Callable[[Tensor], Tensor]] = None,
     batch_initial_conditions: Optional[Tensor] = None,
     cyclic_options: Optional[Dict[str, Union[bool, float, int, str]]] = None,
-    **kwargs,
+    *,
+    ic_generator: Optional[TGenInitialConditions] = None,
+    timeout_sec: Optional[float] = None,
+    return_full_tree: bool = False,
+    retry_on_optimization_warning: bool = True,
+    **ic_gen_kwargs: Any,
 ) -> Tuple[Tensor, Tensor]:
     r"""Generate a set of `q` candidates via cyclic optimization.
 
@@ -407,6 +604,17 @@ def optimize_acqf_cyclic(
             If no initial conditions are provided, the default initialization will
             be used.
         cyclic_options: Options for stopping criterion for outer cyclic optimization.
+        ic_generator: Function for generating initial conditions. Not needed when
+            `batch_initial_conditions` are provided. Defaults to
+            `gen_one_shot_kg_initial_conditions` for `qKnowledgeGradient` acquisition
+            functions and `gen_batch_initial_conditions` otherwise. Must be specified
+            for nonlinear inequality constraints.
+        timeout_sec: Max amount of time optimization can run for.
+        return_full_tree:
+        retry_on_optimization_warning: Whether to retry candidate generation with a new
+            set of initial conditions when it fails with an `OptimizationWarning`.
+        ic_gen_kwargs: Additional keyword arguments passed to function specified by
+            `ic_generator`
 
     Returns:
         A two-element tuple containing
@@ -426,8 +634,7 @@ def optimize_acqf_cyclic(
         >>>     qEI, bounds, 3, 15, 256, cyclic_options={"maxiter": 4}
         >>> )
     """
-    # for the first cycle, optimize the q candidates sequentially
-    candidates, acq_vals = optimize_acqf(
+    opt_inputs = OptimizeAcqfInputs(
         acq_function=acq_function,
         bounds=bounds,
         q=q,
@@ -436,19 +643,32 @@ def optimize_acqf_cyclic(
         options=options,
         inequality_constraints=inequality_constraints,
         equality_constraints=equality_constraints,
+        nonlinear_inequality_constraints=None,
         fixed_features=fixed_features,
         post_processing_func=post_processing_func,
         batch_initial_conditions=batch_initial_conditions,
         return_best_only=True,
+        gen_candidates=gen_candidates_scipy,
         sequential=True,
-        **kwargs,
+        ic_generator=ic_generator,
+        timeout_sec=timeout_sec,
+        return_full_tree=return_full_tree,
+        retry_on_optimization_warning=retry_on_optimization_warning,
+        ic_gen_kwargs=ic_gen_kwargs,
     )
+
+    # for the first cycle, optimize the q candidates sequentially
+    candidates, acq_vals = _optimize_acqf(opt_inputs)
+    q = opt_inputs.q
+    opt_inputs = dataclasses.replace(opt_inputs, q=1)
+    acq_function = opt_inputs.acq_function
+
     if q > 1:
         cyclic_options = cyclic_options or {}
         stopping_criterion = ExpMAStoppingCriterion(**cyclic_options)
         stop = stopping_criterion.evaluate(fvals=acq_vals)
         base_X_pending = acq_function.X_pending
-        idxr = torch.ones(q, dtype=torch.bool, device=bounds.device)
+        idxr = torch.ones(q, dtype=torch.bool, device=opt_inputs.bounds.device)
         while not stop:
             for i in range(q):
                 # optimize only candidate i
@@ -458,20 +678,12 @@ def optimize_acqf_cyclic(
                     if base_X_pending is not None
                     else candidates[idxr]
                 )
-                candidate_i, acq_val_i = optimize_acqf(
-                    acq_function=acq_function,
-                    bounds=bounds,
-                    q=1,
-                    num_restarts=num_restarts,
-                    raw_samples=raw_samples,
-                    options=options,
-                    inequality_constraints=inequality_constraints,
-                    equality_constraints=equality_constraints,
-                    fixed_features=fixed_features,
-                    post_processing_func=post_processing_func,
+                opt_inputs = dataclasses.replace(
+                    opt_inputs,
                     batch_initial_conditions=candidates[i].unsqueeze(0),
-                    return_best_only=True,
+                    sequential=False,
                 )
+                candidate_i, acq_val_i = _optimize_acqf(opt_inputs)
                 candidates[i] = candidate_i
                 acq_vals[i] = acq_val_i
                 idxr[i] = 1
@@ -624,6 +836,8 @@ def optimize_acqf_mixed(
             transformations).
         batch_initial_conditions: A tensor to specify the initial conditions. Set
             this if you do not want to use default initialization strategy.
+        kwargs: kwargs do nothing. This is provided so that the same arguments can
+            be passed to different acquisition functions without raising an error.
 
     Returns:
         A two-element tuple containing
@@ -641,6 +855,7 @@ def optimize_acqf_mixed(
                 "are currently not supported when `q > 1`. This is needed to "
                 "compute the joint acquisition value."
             )
+    _raise_deprecation_warning_if_kwargs("optimize_acqf_mixed", kwargs)
 
     if q == 1:
         ff_candidate_list, ff_acq_value_list = [], []
@@ -727,6 +942,8 @@ def optimize_acqf_discrete(
             a large training set.
         unique: If True return unique choices, o/w choices may be repeated
             (only relevant if `q > 1`).
+        kwargs: kwargs do nothing. This is provided so that the same arguments can
+            be passed to different acquisition functions without raising an error.
 
     Returns:
         A three-element tuple containing
@@ -741,6 +958,7 @@ def optimize_acqf_discrete(
         )
     if choices.numel() == 0:
         raise InputDataError("`choices` must be non-emtpy.")
+    _raise_deprecation_warning_if_kwargs("optimize_acqf_discrete", kwargs)
     choices_batched = choices.unsqueeze(-2)
     if q > 1:
         candidate_list, acq_value_list = [], []
@@ -829,11 +1047,12 @@ def _gen_batch_initial_conditions_local_search(
     max_tries: int = 100,
 ):
     """Generate initial conditions for local search."""
-    tkwargs = {"device": discrete_choices[0].device, "dtype": discrete_choices[0].dtype}
+    device = discrete_choices[0].device
+    dtype = discrete_choices[0].dtype
     dim = len(discrete_choices)
-    X = torch.zeros(0, dim, **tkwargs)
+    X = torch.zeros(0, dim, device=device, dtype=dtype)
     for _ in range(max_tries):
-        X_new = torch.zeros(raw_samples, dim, **tkwargs)
+        X_new = torch.zeros(raw_samples, dim, device=device, dtype=dtype)
         for i, c in enumerate(discrete_choices):
             X_new[:, i] = c[
                 torch.randint(low=0, high=len(c), size=(raw_samples,), device=c.device)
@@ -890,6 +1109,8 @@ def optimize_acqf_discrete_local_search(
             a large training set.
         unique: If True return unique choices, o/w choices may be repeated
             (only relevant if `q > 1`).
+        kwargs: kwargs do nothing. This is provided so that the same arguments can
+            be passed to different acquisition functions without raising an error.
 
     Returns:
         A two-element tuple containing
@@ -897,13 +1118,15 @@ def optimize_acqf_discrete_local_search(
         - a `q x d`-dim tensor of generated candidates.
         - an associated acquisition value.
     """
+    _raise_deprecation_warning_if_kwargs("optimize_acqf_discrete_local_search", kwargs)
     candidate_list = []
     base_X_pending = acq_function.X_pending if q > 1 else None
     base_X_avoid = X_avoid
-    tkwargs = {"device": discrete_choices[0].device, "dtype": discrete_choices[0].dtype}
+    device = discrete_choices[0].device
+    dtype = discrete_choices[0].dtype
     dim = len(discrete_choices)
     if X_avoid is None:
-        X_avoid = torch.zeros(0, dim, **tkwargs)
+        X_avoid = torch.zeros(0, dim, device=device, dtype=dtype)
 
     inequality_constraints = inequality_constraints or []
     for i in range(q):
@@ -931,8 +1154,8 @@ def optimize_acqf_discrete_local_search(
             X0 = X_init[acqvals_init.topk(k=num_restarts, largest=True, dim=0).indices]
 
         # optimize from the best starting points
-        best_xs = torch.zeros(len(X0), dim, **tkwargs)
-        best_acqvals = torch.zeros(len(X0), 1, **tkwargs)
+        best_xs = torch.zeros(len(X0), dim, device=device, dtype=dtype)
+        best_acqvals = torch.zeros(len(X0), 1, device=device, dtype=dtype)
         for j, x in enumerate(X0):
             curr_x, curr_acqval = x.clone(), acq_function(x.unsqueeze(1))
             while True:
